#include "cartridge.h"

Cartridge::Cartridge() {	
	// Set Address Ports as Outputs
	A0ToA7DDR = 0xFF;
	A0ToA7Port = 0x00;
	A8ToA15DDR = 0xFF;
	A8ToA15Port = 0x00;
	
	// Set Control Pins as Outputs
	CTRLDDR = 0xFF;
	CTRLPort = (1<<READ) | (1<<WRITE) | (1<<RESET) | (1<<CS);
	
	// Reset Memory Bank Controller
	reset();
	
	// Setup Timer1
	TCCR1B = 0; // Stop and reset Timer Control Register
	TCNT1 = 0; // Reset Data Register
	OCR1A = OCRA_VAL; // Set Compare-Match Value
	TCCR1B = (1<<WGM12) | (1<<CS10); // Start Timer in CTC-Mode; Prescaler=1
	
	
	// Setup Timer0
	TCCR0 = 0; // Stop and reset Timer Control Register
	OCR0 = OCRA_VAL / 2; // Set Compare-Match Value twice as fast
	TCCR0 = (1<<WGM01) | (1<<CS00); // Start Timer in CTC-Mode; Prescaler=1
	TCNT0 = TCNT1L + 3;  // Sync Timer0 with Timer1, Takes 3 cycles to complete
}

uint8_t Cartridge::readByte(uint16_t address) {
	// Set Data Port as Input
	DataDDR= 0x00;
	DataPort=0x00;
	
	// Set Address ports
	A0ToA7Port= address & 0x00FF;
	A8ToA15Port= address >> 8;
	
	// WR Rises
	CTRLPort|=(1<<WRITE);
	
	// CLK & CS Rises
	while((TIFR & (1<<OCF1A))==0);
	CTRLPort|=(1<<CLK) | (1<<CS);
	TIFR |= (1<<OCF1A);

	// RD falls
	CTRLPort&=~(1<<READ);
	
	// Wait until CLK is at half duty cycle & CS falls
	while((TIFR & (1<<OCF0))==0);
	CTRLPort&=~(1<<CS);
	TIFR |= (1<<OCF0);
	
	// CLK Falls & Read Data
	while((TIFR & (1<<OCF0))==0);
	CTRLPort&=~(0<<CLK);
	TIFR |= (1<<OCF0);
	return DataPin;
}

void Cartridge::writeByte(uint16_t address, uint8_t val) {
	// Set Data Port as Output
	DataDDR= 0xFF;
	
	// CLK Rises
	while((TIFR & (1<<OCF1A))==0);
	CTRLPort|=(1<<CLK);
	TIFR |= (1<<OCF1A);
	
	// RD Rises & Set Address ports
	CTRLPort|=(1<<READ);
	A0ToA7Port= address & 0x00FF;
	A8ToA15Port= address >> 8;
	
	// Wait until CLK is at half duty cycle & CS falls
	while((TIFR & (1<<OCF0))==0);
	CTRLPort&=~(1<<CS);
	TIFR |= (1<<OCF0);
	
	// Wait until CLK falls & WR falls & Set Data Port
	while((TIFR & (1<<OCF0))==0);
	DataPort=val;
	CTRLPort&=~((1<<WRITE) | (1<<CLK));
	TIFR |= (1<<OCF0);
	
	// Wait until CLK is at half duty cycle & WR Rises
	while((TIFR & (1<<OCF0))==0);
	CTRLPort|= (1<<WRITE);
}

void Cartridge::reset() {
	_delay_ms(100);
	CTRLPort&= ~(1<<RESET);
	_delay_ms(100);
	CTRLPort|=(1<<RESET);
}

void* Cartridge::operator new(size_t size) {
	return malloc(size);
}