#include "uart.h"

Uart::Uart(uint32_t baudRate) {
	setBaudRate(baudRate);
}

void Uart::writeChar(const char letter) {
	while (!(UCSRA & (1 << UDRE)));
	UDR = letter;
}

void Uart::writeString(const char* str) {
	for(uint16_t i=0;i!=strlen(str);i++) {
		writeChar(str[i]);
	}
}

char Uart::readChar() {
	while (!(UCSRA & (1<<RXC)));
	return UDR;
}

uint16_t Uart::readString(char* buffer, uint16_t maxLength) {
	char letter = 0;
	uint16_t count = 0;
	do {
		letter = readChar();
		count++;
		realloc(buffer, count * sizeof(letter) );
		buffer[count - 1] = letter;
	} while(letter != 0 && count != maxLength + 1);
	if(count == maxLength + 1) {
		realloc(buffer, count * sizeof(letter) );
		buffer[count - 1] = 0x00;
	}
	return count;
}

void Uart::setBaudRate(uint32_t baudRate) {
	uint16_t ubrr=(uint16_t) ((((F_CPU) + (baudRate * 8UL))/(baudRate * 16UL)) - 1UL);
	if ((100UL * (F_CPU)) > ((16UL * ((ubrr) + 1UL)) * (100UL * (baudRate) + (baudRate) * (BAUD_TOL)))) {
		ubrr=(uint16_t) ((((F_CPU) + (baudRate * 4UL))/(baudRate * 8UL)) - 1UL); // 2X
		UCSRA |= (1 << U2X);
	} else if ((100UL * (F_CPU)) < ((16UL * ((ubrr) + 1UL)) * (100UL * (baudRate) - (baudRate) * (BAUD_TOL)))) {
		UCSRA &= ~(1 << U2X);
	}
	
	UBRRH = (uint8_t) (ubrr & 0xFF00) >> 8;
	UBRRL = (uint8_t) ubrr;

	UCSRB = (1<<RXEN) | (1<<TXEN);
	UCSRC = (1 << URSEL)|(1<<UCSZ1) | (1<<UCSZ0);
}


uint16_t Uart::readWord() {
	uint16_t a = (uint16_t) readChar();
	uint16_t b = (uint16_t) readChar();
	return ((a << 8) | (b));
}

uint32_t Uart::readLong() {
	uint32_t a = (uint32_t) readChar();
	uint32_t b = (uint32_t) readChar();
	uint32_t c = (uint32_t) readChar();
	uint32_t d = (uint32_t) readChar();
	return ((a << 24) | (b << 16) | (c << 8) | (d));
}


void* Uart::operator new(size_t size) {
	return malloc(size);
}