#include "main.h"

int main(void)
{
	// Reserve Memory
	uint8_t symbol = 0;
	uint16_t address = 0;
	uint16_t length = 0;
	
	// Initialize Drivers
	Uart uart(CONTROL_BAUD);
	Cartridge cartridge;
	SerialController serial(&uart, &cartridge);
	
	// Send READY
	uart.writeString("READY");
	
	while(1) {
		symbol = uart.readChar();
		switch(symbol) {
			case 0x00: // CPU Reset
				wdt_enable(WDTO_500MS);
				while(1) {} // Endless loop to trigger watchdog
				// Never reached anyway -> No break (:
			case 0x01: // Set BaudRate
				uart.setBaudRate(uart.readLong());
				break;
			case 0x02: // Reset MBC
				cartridge.reset();
				uart.writeString("OK");
				break;
			case 0x03: // Read Byte
				serial.readByte(uart.readWord());
				break;
			case 0x04: // Write Byte
				serial.writeByte(uart.readWord());
				break;
			case 0x05: // Read n Bytes
				address = uart.readWord();
				length = uart.readWord();
				serial.readBlock(address, length);
				break;
			case 0x06: // Write n Bytes
				address = uart.readWord();
				length = uart.readWord();
				serial.writeBlock(address, length);
				break;
			case 0x07: // Selftest
				uart.writeChar(uart.readChar());
				break;
		}
	}
}