#include "serialController.h"

SerialController::SerialController(Uart* uartDriver, Cartridge* cartridgeDriver) {
	uart = uartDriver;
	cartridge = cartridgeDriver;
}

void SerialController::readBlock(uint16_t start, uint16_t length) {
	for(uint16_t addr = start; addr < (start + length); addr++) {
		uart->writeChar(cartridge->readByte(addr));
	}
}

void SerialController::readByte(uint16_t address) {
	uart->writeChar(cartridge->readByte(address));
}

void SerialController::writeBlock(uint16_t start, uint16_t length) {
	for(uint16_t addr = start; addr < (start + length); addr++) {
		cartridge->writeByte(addr ,uart->readChar());
	}
}

void SerialController::writeByte(uint16_t address) {
	cartridge->writeByte(address ,uart->readChar());
}


void SerialController::setBaud(uint32_t baudRate) {
	uart->setBaudRate(baudRate);
}

void* SerialController::operator new(size_t size) {
	return malloc(size);
}

Uart* SerialController::getUartDriver() {
	return uart;
}

Cartridge* SerialController::getCartridgeDriver() {
	return cartridge;
}
