// A Serial Controller that handles Cartridge Output/ Input and forwards it to the Serial Line or vice versa

#ifndef SERIALCONTROL_H_
#define SERIALCONTROL_H_
#include "drivers/uart.h"
#include "drivers/cartridge.h"
#include <stdlib.h>

class SerialController {
	private:
		Uart* uart; // UART Driver
		Cartridge *cartridge; // Cartridge Driver
		
	public:
		void readBlock(uint16_t start, uint16_t length); // Reads a Block of Memory with a length from the Cartridge and transmits it via Serial Line
		void writeBlock(uint16_t start, uint16_t length); // Writes a Block of Memory with a length from the Serial Line to the Cartridge
		void readByte(uint16_t address); // Reads a Byte from the Cart at address and transmits it via Serial Line
		void writeByte(uint16_t address); // Writes a byte from the Serial Line to the Cartridge at address
		void setBaud(uint32_t baudRate); // Sets the BaudRate
		Uart* getUartDriver(); // Returns the UART Driver
		Cartridge* getCartridgeDriver(); // Returns the Game Boy Cartridge Drivers
		void* operator new(size_t size);
		SerialController(Uart* uartDriver, Cartridge* cartridgeDriver);
};



#endif /* SERIALCONTROL_H_ */