# Game Boy Cartreader & -Writer
[![Build Status](https://ci.phenomic.net/job/git.phenomic.net/job/Gameboy-Color-Cart-Reader/job/master/badge/icon?style=flat-square)](https://ci.phenomic.net/job/git.phenomic.net/job/Gameboy-Color-Cart-Reader/job/master/)[![Build Status Firmware](https://ci.phenomic.net/job/git.phenomic.net/job/Gameboy-Color-Cart-Reader/job/master/badge/icon?config=firmwarebuild&style=flat-square)](https://ci.phenomic.net/job/git.phenomic.net/job/Gameboy-Color-Cart-Reader/job/master/)[![Build Status Windows](https://ci.phenomic.net/job/git.phenomic.net/job/Gameboy-Color-Cart-Reader/job/master/badge/icon?config=winbuild&style=flat-square)](https://ci.phenomic.net/job/git.phenomic.net/job/Gameboy-Color-Cart-Reader/job/master/)

This is a Game Boy Cartreader and -Writer for the Atmel AVR ATmega32 Microcontroller. It is able to read ROM, RAM Saves and the RTC Registers on Game Boy Cartridges. It also is able to write to the RTC Registers and RAM Saves. 

[Go to the Repository](https://git.phenomic.net/PHENOM/Gameboy-Color-Cart-Reader)

![Image of Cartreader](images/cartreader.jpg)

## Table of Contents

- [Game Boy Cartreader & -Writer](#game-boy-cartreader---writer)
  - [Table of Contents](#table-of-contents)
  - [Motivation](#motivation)
  - [Circuitboard](#circuitboard)
  - [Getting Started](#getting-started)
    - [Required Hardware](#required-hardware)
    - [Prerequisites](#prerequisites)
    - [Compiling the Firmware](#compiling-the-firmware)
    - [Flashing the Firmware](#flashing-the-firmware)
    - [Installation of the Client](#installation-of-the-client)
  - [Usage](#usage)
    - [Serial Commandlist](#serial-commandlist)
    - [Go Cartreader Client](#go-cartreader-client)
  - [Dependencies](#dependencies)
  - [Game Boy Documentation used](#game-boy-documentation-used)
  - [License](#license)

## Motivation

I started this project in 2014 and working on it further and overhauling it an will be used in combination with the [Game Boy Emulator](https://git.phenomic.net/PHENOM/GameBoyEmulator).

## Circuitboard

**TODO:** Schematics and board layout will follow in later commits, once I've reworked the board layout for a more production ready state.

## Getting Started

### Required Hardware

- AVR ISP Programmer (In-System-Programmer) to program the Microcontroller
- Game Boy Cartreader & -Writer Circuit Board
- USB 2.0 Cable Type B for the Cartreader

### Prerequisites

In order to use this project you need to have all the required Hardware and have either [AVR Toolchain](https://www.microchip.com/mplab/avr-support/avr-and-arm-toolchains-c-compilers) installed for 8-Bit Controllers or use [AtmelStudio](https://www.microchip.com/mplab/avr-support/atmel-studio-7) to compile the project. Afterwards you want to use a Flashing tool such as avrdude to write the firmware on the controller. You also need to have the [Go Binaries](https://golang.org/dl/) installed and having set the [$GOPATH](https://github.com/golang/go/wiki/SettingGOPATH) environment variable to your Go workspace.

### Compiling the Firmware

```bash
sudo apt-get install gcc-avr avr-libc
cd Game-Boy-Cartreader-Firmware
make
```

### Flashing the Firmware

```bash
sudo apt-get install avrdude
cd Game-Boy-Cartreader-Firmware

# Writing the Firmware
avrdude -c <programmer> -P <port> -p m32 -U flash:w:game-boy-cartreader-firmware.hex
avrdude -c <programmer> -P <port> -p m32 -U eeprom:w:game-boy-cartreader-firmware_eeprom.hex

# Setting the fuses
avrdude -c <programmer> -P <port> -p m32 -U hfuse:w:0xC1:m
avrdude -c <programmer> -P <port> -p m32 -U lfuse:w:0xFF:m
```

If you need a comprehensive guide on how to use avrdude please take a look [here](http://www.ladyada.net/learn/avr/avrdude.html).

### Installation of the Client

Currently only Windows is supported. Cross-Platform support will follow.

```bash
# Clone Project and receive dependencies
go get git.phenomic.net/phenom/gameboy-color-cart-reader/...

# Install Client
cd $GOPATH/src/git.phenomic.net/phenom/gameboy-color-cart-reader
go install ./...
```

## Usage

### Serial Commandlist

This describes the Protocol being used for Communication over the serial line. When rebooting the device the Baudrate is set to **9600 Symbols per second** on reset.

```
+---------+-----------------------------------+-------------------------------------------------------------------------------+
| Command | Parameter List (Length Name, ...) |                                  Description                                  |
+---------+-----------------------------------+-------------------------------------------------------------------------------+
| 0x00    | -                                 | Trigger CPU Reset and print READY(0x52 0x45 0x41 0x44 0x59) after back Online |
| 0x01    | 0x04 Baudrate                     | Set Baudrate (Initially 9600)                                                 |
| 0x02    | -                                 | Reset Memory Banking Controller and print OK (0x4F 0x4B) after done           |
| 0x03    | 0x02 Address                      | Read Byte from Address                                                        |
| 0x04    | 0x02 Address, 0x01 Value          | Write Byte on Address                                                         |
| 0x05    | 0x02 Address, 0x02 Length         | Read n Bytes from Address to (Address + Length)                               |
| 0x06    | 0x02 Address, 0x02 Length         | Writes n Bytes from Serial from Address to (Address + Length)                 |
| 0x07    | 0x01 Value                        | Selftest: Returns the Value over Serial                                       |
+---------+-----------------------------------+-------------------------------------------------------------------------------+
```

### Go Cartreader Client

**TODO:** This will describe how to use the Client for the Cartreader device.

## Dependencies

- [Serial](https://github.com/tarm/serial)

## Game Boy Documentation used

- [Cartridge Header](http://gbdev.gg8.se/wiki/articles/The_Cartridge_Header)
- [Memory Bank Controllers](http://gbdev.gg8.se/wiki/articles/Memory_Bank_Controllers)
- [Emulating a GameBoy Cartridge with an STM32F4](https://dhole.github.io/post/gameboy_cartridge_emu_1/)
- [Game Boy CPU Manual - Page 137-138: Typical Timings](http://marc.rawer.de/Gameboy/Docs/GBCPUman.pdf)

## License

Distributed under the MIT License. See `LICENSE` for more information.